
from __future__ import division

import sys
import cairo
import pygame
import time


class Window(object):
    fps = 50

    background = (0, 0, 0)

    def draw(self, ctx):
        pass

    def react(self, event):
        pass

    def run(self, w, h, caption=None, fullscreen=False):
        pygame.display.init()
        pygame.font.init()

        flags = pygame.FULLSCREEN if fullscreen else 0
        screen = pygame.display.set_mode((w, h), flags)
        if caption: pygame.display.set_caption(caption)

        if sys.byteorder == 'little':
            screen.set_masks((0x00FF0000, 0x0000FF00, 0x000000FF, 0x00000000))
        else:
            # TODO: this might be wrong
            screen.set_masks((0x0000FF00, 0x00FF0000, 0xFF000000, 0x00000000))

        def draw_internal():
            screen.fill(self.background)

            # screendata must not be in scope anymore when doing flip()
            screendata = pygame.surfarray.pixels2d(screen).data
            csurf = cairo.ImageSurface.create_for_data(
                    screendata, cairo.FORMAT_RGB24, w, h)
            ctx = cairo.Context(csurf)
            self.draw(ctx)

        self.begin = time.time()
        self.running = True
        self.clock = pygame.time.Clock()
        while self.running:
            draw_internal()
            pygame.display.flip()
            if not self.running: return

            for event in pygame.event.get():
                self.react(event)
                if not self.running: return

            self.clock.tick(self.fps)


class Timeline(object):
    def __init__(self, window, speed):
        self.begin = None
        self.finished = False
        self.pause = None
        self.speed = speed
        self.window = window
        window.draw = self.draw_internal
        window.react = self.react_internal

    def draw(self, ctx, now):
        pass

    def react(self, event):
        pass

    def draw_internal(self, ctx):
        if self.begin is None: self.begin = time.time()
        now = self.pause if self.pause is not None else (time.time() - self.begin) / self.speed
        if now < 0:
            now = 0
            self.pause = 0
            self.speed = abs(self.speed)
        self.draw(ctx, now)

    def react_internal(self, event):
        if self.react(event): return
        if event.type == pygame.QUIT:
            self.window.running = False
        if hasattr(event, 'mod'):
            mod = event.mod & (pygame.KMOD_NUM - 1)
        if event.type == pygame.KEYUP and mod in (pygame.KMOD_LALT, pygame.KMOD_RALT) and event.key == pygame.K_s:
            self.finished = True
            self.window.running = False
        if event.type == pygame.KEYUP and mod in (pygame.KMOD_LALT, pygame.KMOD_RALT) and event.key == pygame.K_q:
            self.window.running = False
        if event.type == pygame.KEYDOWN and mod == 0 and event.key == pygame.K_p:
            if self.pause is None:
                self.pause = (time.time() - self.begin) / self.speed
            else:
                self.begin = time.time() - self.pause * self.speed
                self.pause = None
        if event.type == pygame.KEYDOWN and mod == 0 and event.key == pygame.K_SPACE:
            if self.pause is None:
                self.pause = (time.time() - self.begin) / self.speed
            else:
                self.pause = int(self.pause) + 1
        # TODO +1 krok, ale animovany (ergo odpauzuj ale pauzni na dalsom celom kroku)
        rychlomenice = { pygame.K_MINUS: 1.25, pygame.K_EQUALS: 0.8, pygame.K_r: -1 }
        if event.type == pygame.KEYDOWN and rychlomenice.get(event.key):
            was_unpaused = self.pause is None
            if was_unpaused:
                self.pause = (time.time() - self.begin) / self.speed
            self.speed *= rychlomenice[event.key]
            if was_unpaused:
                self.begin = time.time() - self.pause * self.speed
                self.pause = None


def draw_test(ctx):
    ctx.select_font_face('monospace')
    ctx.set_font_size(16)
    font_height = ctx.text_extents('0')[3]
    now = time.time() - window.begin

    ctx.set_source_rgb(1, 0.5, 0)
    ctx.arc(70 + now*20, 70 + now*5.5, 47, 0, 3.8)
    ctx.fill()
    ctx.arc(70 + now*20, 70 + now*5.5, 47, 4, -0.2)
    ctx.fill()

    ctx.save()
    ctx.set_source_rgb(1, 1, 1)
    ctx.translate(0, font_height)
    ctx.show_text('%.3f' % window.clock.get_fps())
    ctx.restore()


if __name__ == '__main__':
    window = Window()
    window.draw = draw_test
    window.run(800, 600, 'Hello')
